@@ -1,3 +1,5 @@
+#define PKTSIZE SEGSIZE+4       /* should be moved to tftp.h */
+
 int synchnet(int);
 struct tftphdr *r_init(void);
 struct tftphdr *w_init(void);
@@ -5,3 +7,5 @@ int readit(FILE *file, struct tftphdr **dpp, int convert);
 int writeit(FILE *file, struct tftphdr **dpp, int ct, int convert);
 void read_ahead(FILE *file, int convert /* if true, convert to ascii */);
 int write_behind(FILE *file, int convert);
+
+void mysignal(int, void (*func)(int));
